// PLUGIN.H
//
// This header file defines the interfaces for cdrLabel graphic
// and file filter plugins.
//
// Any questions or comments, please feel free to contact me at:
//    Earle F. Philhower, III  <earle@ziplabel.com>

#ifndef _PLUGIN_H
#define _PLUGIN_H

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */

#define EXPORT __declspec(dllexport)

// The version this plugin was compiled to
//#define PLUGINVERSION 1
#define PLUGINVERSION 2

//--------------------------------------------------------------------
// File filter plugin structures and functions


#define MAXTITLE MAX_PATH // The maximum length of a plugin's friendly name
#define NAMELEN 64        // Maximum length of a file's name in a MINIFILEINFO
#define COMMLEN 16        // Maximum length of a comment in a MINIFILEINFO
  
  // The alignment of each file is individually specified...
  typedef enum FORMATTYPE {FMT_LEFT=0, FMT_CENTER, FMT_RIGHT};

  // One per directory level
  typedef struct MINIZIPINFO {
    unsigned int numfiles;   // Number of files in the attached MINIFILEINFO
    struct MINIFILEINFO *fi; // The actual files
  } MINIZIPINFO;
  
  // One per file
  typedef struct MINIFILEINFO {
    char name[NAMELEN];    // File name, C string, ANSI text, 0 terminated
    char comm[COMMLEN];    // Comment (i.e. "12KB"or "1:20") 
    __int64 size;          // File size in bytes (modified V7.1 to i64 from DWORD)
    unsigned char isdir;   // BOOLEAN, is this a subdirectory?
    unsigned char showdir; // If ISDIR==TRUE => should dir be open on label?
    unsigned char format;  // FORMATTYPE: String orientation
    MINIZIPINFO *zi;       // if ISDIR==TRUE => subdirectory ZIPINFO
                           // if ISDIR==FALSE => NULL
  } MINIFILEINFO;
  
  // One per plugin DLL call
  typedef struct MINIFILEPARAM {
    unsigned int version;  // [in] Version of the plugin interface used.
                           //   Set by cdrLabel on call to FileFilterInitialize
    void *user;            // [out] Free for plugin to use (like a LPARAM)
    HKEY userKey;          // [in] Registry key for storing your preferences
                           //      Valid at all times.  Do not close!
    char title[MAXTITLE];  // [out] The name to be displayed in menus for
                           //  this plugin
    
    // ONLY FOLLOWING FUNCTIONS MAY BE USED FOR PASSING DATA BACK TO CDRLABEL!
    // malloc(), GlobalAlloc(), etc. are not allowed to be returned!

    // Creates a minizip structure of the specified number of entries and
    // returns a pointer.  On failute returns a NULL.
    // !! You are responsible for freeing this structure if you
    //    do not return it from your FileFilterExecute() function.
    MINIZIPINFO *(*CreateMiniZipInfo)(unsigned int numEntries);

    // Resizes a minizip structure (previously allocated via CreateMiniZip()
    // or passed into the DLL in the FileFilterExecute() function) to the
    // specified number of entries.  On failure returns NULL.
    // !! Does not clear out any added entries.
    MINIZIPINFO *(*ResizeMiniZipInfo)(MINIZIPINFO *miniZip,
				      unsigned int newEntries);

    // Frees minizip structure allocated by CreateMiniZip or passed to the DLL
    // !! All minizips that are passed in or created by the DLL, but not
    //    returned to cdrLabel in the return value of FileFilterExecute,
    //    must be freed by the DLL.
    void (*FreeMiniZipInfo)(MINIZIPINFO *miniZip);
  } MINIFILEPARAM;



  // The Initialize() function is called when cdrLabel is starting up.
  // First, the DLL should check that it supports the version number passed in
  // the param structure.  If it does not then it should return FALSE and
  // it will not be loaded by cdrLabel.
  // If the version is OK the DLL should then strncpy() it's name to the
  // title structure.
  // Next it can interrogate the HKEY userKey passed(the key is already opened)
  // and load any settings from the registry.
  // A pointer to these settings can then be stored in the void *user, or
  // they can just be declared static and kept in your DLL's space.
  // Once all is done the DLL should then return TRUE to have cdrLabel add it
  // to the list.
  EXPORT int FileFilterInitialize( MINIFILEPARAM *param );
  
  // The UserInterface() function is called when the user selects the plugin
  // name from the menu.  It should display a dialog box to get user settings.
  // One setting that it should always have is an Enable/Disable checkbox, so
  // that a user may disable a plugin without having to remove it from the
  // plugins directory.
  // ** The UserInterface() call does NOT need to be called, *ever*.       **
  // ** Safe default values should be set in your initialize() routine and **
  // ** your execute() function should work without ever running UI().     **
  EXPORT void FileFilterUserInterface( MINIFILEPARAM *param, HWND parent );
  
  // Execute() is called after the entire CD has been scanned by cdrLabel.
  // They are passed a pointer to a MINIZIPINFO and the root path of the CD.
  // The Execute() function can perform any change, add or subtract files,
  // rename existing entries, etc. and returns a pointer to the modified
  // structure. 
  // If the plugin does not want to change any entries it can return the
  // pointer that was passed into it.
  // label[] is the spine label and labelSize is the maximum length
  // (including terminating 0) of the array.
  EXPORT MINIZIPINFO *FileFilterExecute( MINIFILEPARAM *param,
					 MINIZIPINFO *files,
					 char *label,
					 unsigned int labelSize,
					 char *rootPath );
  
  // Unitialize() is called when cdrLabel is shutting down.  You should save
  // any changed settings in the HKEY passed in, free any buffers, etc.
  EXPORT void FileFilterUninitialize( MINIFILEPARAM *param );


//--------------------------------------------------------------------
// Graphic filter plugin structures and functions


  // Structure containing all the info you get about a graphic to be filtered
  // All images are transferred as packed RGB, bottom- to top-line (like DIBs)
  // The data will always be 3-byte packed RGB format, and always bottom-to-top
  // format [i.e. bottom_line = bits; top_line = (bits+lineWidth*(height-1)); ]
  typedef struct MINIGRAPHIC {
    unsigned int width;     // Width in pixels
    unsigned int height;    // Height in pixels
    unsigned int lineWidth; // Number of *bytes* from one line to the next
    unsigned char *bits;    // The actual bits, stored in Windows 24-bit RGB
                            // format, bottom-to-top like a DIB
  } MINIGRAPHIC;
  
  // Structure that is passed into the DLL
  typedef struct MINIGRAPHICPARAM {
    unsigned int version; // [in] Version of the plugin interface used.
                          // Set by cdrLabel on call to GraphicFilterInitialize
    void *user;           // [out] Free for plugin to use (like a LPARAM)
    HKEY userKey;         // [in] Registry key for storing your preferences
                          //      Always valid.  Do not close!
    char title[MAXTITLE]; // [out] Name to display in menus for this plugin
    
    // ONLY FOLLOWING FUNCTIONS MAY BE USED FOR PASSING DATA BACK TO CDRLABEL!
    // malloc(), GlobalAlloc(), etc. are not allowed.
    
    // Returns pointer to blank MINIGRAPHIC of the specified width and height.
    // Returns NULL on error.  Any MINIGRAPHICS that are created but not
    // returned by the UserInterface() function should be freed before
    // returning.
    MINIGRAPHIC *(*CreateMiniGraphic)( unsigned int width,
				       unsigned int height );

    // Frees the specified minigraphic
    void (*FreeMiniGraphic)( MINIGRAPHIC *graphic );

    // ** Following functions only legal during DLL's UserInterface call **
    // Updates the on-screen display of the image with passed in minigraphic
    void (*PreviewMiniGraphic)( MINIGRAPHIC *graphic );

    // Gets the display rectangle (MM_LOENGLISH coordinates) of the graphic
    // Note that this display rectangle has NO relation to the actual
    // graphic size in pixels, it can be stretched or shrunk
    void (*GetDisplayRect)( RECT *rect );

    // Sets the display rectangle (MM_LOENGLISH coordinates) of the graphic
    // Note that this display rectangle has NO relation to the actual
    // graphic size in pixels, it can be stretched or shrunk
    void (*SetDisplayRect)( RECT *rect );

  } MINIGRAPHICPARAM;

  
  // The Initialize() function is called when cdrLabel is starting up.
  // First, the DLL should check that it supports the version number passed in
  // the param structure.  If it does not then it should return FALSE and
  // it will not be loaded by cdrLabel.
  // If the version is OK the DLL should then strncpy() it's name to the
  // title structure.
  // Next it can interrogate the HKEY userKey passed(the key is already opened)
  // and load any settings from the registry.
  // A pointer to these settings can then be stored in the void *user, or
  // they can just be declared static and kept in your DLL's space.
  // Once all is done the DLL should then return TRUE to have cdrLabel add it
  // to the list.
  EXPORT int GraphicFilterInitialize( MINIGRAPHICPARAM *param );
  
  // The UserInterface is the function that (optionally) shows a dialog box and
  // does the image processing or generation.  It is passed in the parent of
  // the dialog box to be created and a minigraphic of the currently selected
  // image.
  // It may call param->Preview() as often as it wants to update the
  // displayed image prior to returning a final, modified MINIGRAPHIC.
  // If no processing should be done, it should just return the minigraphic
  // that was passed into it.  If it does processing into a new minigraphic,
  // then the minigraphic that was passed in should be freed.  If it processes
  // the image in-place then it can just return the modified minigraphic that
  // was passed in.
  EXPORT MINIGRAPHIC *GraphicFilterUserInterface( MINIGRAPHICPARAM *param,
						  HWND parent,
						  MINIGRAPHIC *graphic );
  
  // Unitialize() is called when cdrLabel is shutting down.  You should save
  // any changed settings in the HKEY passed in, free any buffers, etc.
  EXPORT void GraphicFilterUninitialize( MINIGRAPHICPARAM *param );
  

#ifdef __cplusplus
}
#endif  /* __cplusplus */


#endif // #ifndef _PLUGIN_H
